%==========================================================================
%  Function: draw_tractor()
%       Plot a tractor with trailer to the current Figure.
%
%  Synopsis:
%      draw_tractor(x,u,L1,L2)
%
%  Input Parameter:
%      -x: State 4x1 x = [x_T, y_T, beta, gamma]^T
%      -u: Input 1x1 u = alpha
%      -L1: length Tractor
%      -L2: length Trailer
%       
%  Author: Peter Hofmeier
%  Systems control and optimization laboratory, 
%  Department of Microsystems Engineering (IMTEK),
%  University of Freiburg
% 
%  File Created: 07.06.2017
%==========================================================================
function draw_tractor(x,u,L1,L2)

% Konstante
linewidth = 2;

Lwb = 2.5;  % Lnge des Radstanges Tractor back
Lwf = 2.0;  % Lnge des Radstandes Tractor front
lwrB=1.6;  % Lnge Rad back
wwrB=1.2;  % Breite Rad Back
lwrF=1.1;  % Lnge Rad Front
wwrF=0.8;  % Breite Rad Front

LTr=1.25; % Lnge Randstand Anhnger
lTrwr=1.1;  % Lnge Rad Anhnger
wTrwr=0.8;  % Breite Rad Anhnger

% Berechen Positionen des Traktores
pH  = x(1:2);  % Back position achsenmittelpunkt
pF  = [pH(1)+L1*cos(x(3))  pH(2)+L1*sin(x(3))];  % Front position achsenmittelpunkt

% Berechnen der Rder des Traktors
pW1 = [pH(1)-Lwb*sin(x(3)) pH(2)+Lwb*cos(x(3))]; % Back wheel 1 position
pW11= [pW1(1)+ lwrB*cos(x(3)),pW1(2)+ lwrB*sin(x(3))];
pW12= [pW1(1)- lwrB*cos(x(3)),pW1(2)- lwrB*sin(x(3))];
pW13= [pW1(1)-wwrB*sin(x(3))- lwrB*cos(x(3)),pW1(2)+wwrB*cos(x(3)) - lwrB*sin(x(3))];
pW14= [pW1(1)-wwrB*sin(x(3))+ lwrB*cos(x(3)),pW1(2)+wwrB*cos(x(3)) + lwrB*sin(x(3))];

pW2 = [pH(1)+Lwb*sin(x(3)) pH(2)-Lwb*cos(x(3))]; % Back wheel 2 position
pW21= [pW2(1)+ lwrB*cos(x(3)),pW2(2)+ lwrB*sin(x(3))];
pW22= [pW2(1)- lwrB*cos(x(3)),pW2(2)- lwrB*sin(x(3))];
pW23= [pW2(1)+wwrB*sin(x(3))- lwrB*cos(x(3)),pW2(2)-wwrB*cos(x(3)) - lwrB*sin(x(3))];
pW24= [pW2(1)+wwrB*sin(x(3))+ lwrB*cos(x(3)),pW2(2)-wwrB*cos(x(3)) + lwrB*sin(x(3))];

pW3 = [pF(1)-Lwf*sin(x(3)+u) pF(2)+Lwf*cos(x(3)+u)]; % Front wheel 1 position
pW31= [pW3(1)+ lwrF*cos(x(3)+u),pW3(2)+ lwrF*sin(x(3)+u)];
pW32= [pW3(1)- lwrF*cos(x(3)+u),pW3(2)- lwrF*sin(x(3)+u)];
pW33= [pW3(1)-wwrF*sin(x(3)+u)- lwrF*cos(x(3)+u),pW3(2)+wwrF*cos(x(3)+u) - lwrF*sin(x(3)+u)];
pW34= [pW3(1)-wwrF*sin(x(3)+u)+ lwrF*cos(x(3)+u),pW3(2)+wwrF*cos(x(3)+u) + lwrF*sin(x(3)+u)];

pW4 = [pF(1)+Lwf*sin(x(3)+u) pF(2)-Lwf*cos(x(3)+u)]; % Front wheel 2 position
pW41= [pW4(1)+ lwrF*cos(x(3)+u),pW4(2)+ lwrF*sin(x(3)+u)];
pW42= [pW4(1)- lwrF*cos(x(3)+u),pW4(2)- lwrF*sin(x(3)+u)];
pW43= [pW4(1)+wwrF*sin(x(3)+u)- lwrF*cos(x(3)+u),pW4(2)-wwrF*cos(x(3)+u) - lwrF*sin(x(3)+u)];
pW44= [pW4(1)+wwrF*sin(x(3)+u)+ lwrF*cos(x(3)+u),pW4(2)-wwrF*cos(x(3)+u) + lwrF*sin(x(3)+u)];

% Berechnen der Position des Anhngers
pTrH = [x(1)-L2.*cos(x(4)); ...
          x(2)-L2.*sin(x(4))]; % BackPosition Trailer

% Berechnen der Rder des Traktors
pTrW1= [pTrH(1)-LTr*sin(x(4)) pTrH(2)+LTr*cos(x(4))]; % Trailer wheel 1 position
pTrW11= [pTrW1(1)+ lTrwr*cos(x(4)),pTrW1(2)+ lTrwr*sin(x(4))];
pTrW12= [pTrW1(1)- lTrwr*cos(x(4)),pTrW1(2)- lTrwr*sin(x(4))];
pTrW13= [pTrW1(1)-wTrwr*sin(x(4))- lTrwr*cos(x(4)),pTrW1(2)+wTrwr*cos(x(4)) - lTrwr*sin(x(4))];
pTrW14= [pTrW1(1)-wTrwr*sin(x(4))+ lTrwr*cos(x(4)),pTrW1(2)+wTrwr*cos(x(4)) + lTrwr*sin(x(4))];

pTrW2= [pTrH(1)+LTr*sin(x(4)) pTrH(2)-LTr*cos(x(4))]; % Trailer wheel 1 position
pTrW21= [pTrW2(1)+ lTrwr*cos(x(4)),pTrW2(2)+ lTrwr*sin(x(4))];
pTrW22= [pTrW2(1)- lTrwr*cos(x(4)),pTrW2(2)- lTrwr*sin(x(4))];
pTrW23= [pTrW2(1)+wTrwr*sin(x(4))- lTrwr*cos(x(4)),pTrW2(2)-wTrwr*cos(x(4)) - lTrwr*sin(x(4))];
pTrW24= [pTrW2(1)+wTrwr*sin(x(4))+ lTrwr*cos(x(4)),pTrW2(2)-wTrwr*cos(x(4)) + lTrwr*sin(x(4))];


% Plot frame Tractor
line([pH(1) pF(1)],[pH(2) pF(2)],'Color','black','LineWidth',linewidth)
hold on
line([pW1(1) pW2(1)],[pW1(2) pW2(2)],'Color','black','LineWidth',linewidth)
line([pW3(1) pW4(1)],[pW3(2) pW4(2)],'Color','black','LineWidth',linewidth)

% Plot wheels Tractor
fill([pW11(1),pW12(1),pW13(1),pW14(1)],[pW11(2),pW12(2),pW13(2),pW14(2)],'black')
fill([pW21(1),pW22(1),pW23(1),pW24(1)],[pW21(2),pW22(2),pW23(2),pW24(2)],'black')
fill([pW31(1),pW32(1),pW33(1),pW34(1)],[pW31(2),pW32(2),pW33(2),pW34(2)],'black')
fill([pW41(1),pW42(1),pW43(1),pW44(1)],[pW41(2),pW42(2),pW43(2),pW44(2)],'black')

% Plot Frame Trailer
line([pH(1) pTrH(1)],[pH(2) pTrH(2)],'Color','g','LineWidth',linewidth)
line([pTrW1(1) pTrW2(1)],[pTrW1(2) pTrW2(2)],'Color','g','LineWidth',linewidth)

% Plot wheels Trailer
fill([pTrW11(1),pTrW12(1),pTrW13(1),pTrW14(1)],[pTrW11(2),pTrW12(2),pTrW13(2),pTrW14(2)],'g')
fill([pTrW21(1),pTrW22(1),pTrW23(1),pTrW24(1)],[pTrW21(2),pTrW22(2),pTrW23(2),pTrW24(2)],'g')

end